package net.tekiton.minesweeper;

import java.awt.*;
import java.util.HashMap;
import java.util.Locale;

import javax.swing.*;
import javax.swing.border.*;

@SuppressWarnings("serial")
public class Minesweeper extends JFrame{

	static int FIELD_WIDTH;
	static int FIELD_HEIGHT;
	static int FIELD_SIZE;
	static int FIELD_BUTTON_SIZE;
	static int MINE_QUANTITY;
	static int tempMineQuantity;
	static int countOpen = 0;
	static int[][] field = null;
	static boolean[][] isCheck = null;
	static boolean[][] isFlag = null;
	static boolean isOver = false;
	static boolean isStart = false;
	static JButton[][] fieldButton = null;
	static JButton refreshButton = null;
	static JLabel timeField = new JLabel();
	static JLabel quantityField = new JLabel();
	static PlayTime t = new PlayTime();
	ClassLoader cl = (new Dummy()).getClass().getClassLoader();

	static Locale systemLocale = Locale.getDefault();
	static HashMap<String,String> labelName = new HashMap<String,String>();

/*
	public static void main(String[] args) {
		Minesweeper frame = new Minesweeper(20, 15);
		frame.setVisible(true);
	}
*/

	Minesweeper(int width, int height){
		this(width, height, (width-1)*(height-1)/4, 22);
	}

	Minesweeper(int width, int height, int quantity){
		this(width, height, quantity, 22);
	}

	Minesweeper(int width, int height, int quantity,int buttonSize){
		FIELD_WIDTH = width;
		FIELD_HEIGHT = height;
		FIELD_SIZE = width * height;
		MINE_QUANTITY = quantity;
		tempMineQuantity = MINE_QUANTITY;
		FIELD_BUTTON_SIZE = buttonSize;

		field = new int[FIELD_HEIGHT+2][FIELD_WIDTH+2];
		isCheck = new boolean[FIELD_HEIGHT+2][FIELD_WIDTH+2];
		isFlag = new boolean[FIELD_HEIGHT+2][FIELD_WIDTH+2];
		fieldButton = new JButton[FIELD_HEIGHT+2][FIELD_WIDTH+2];

		if(systemLocale.getLanguage().equals(new Locale("ja", "", "").getLanguage())){
			labelName.put("title", "マインスイーパ");
			labelName.put("reset", "リセット");
			labelName.put("resize", "盤面の変更");
		}else{
			labelName.put("title", "Minesweeper");
			labelName.put("reset", "Reset");
			labelName.put("resize", "Resize");
		}

		setTitle(labelName.get("title"));

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
		setVisual();

		getContentPane().add(mainPane(), BorderLayout.CENTER);
		setIconImage(new ImageIcon(cl.getResource("bomb.png")).getImage());
		pack();
		setLocationRelativeTo(null);

		resetField();
	}


	JPanel mainPane(){

		JPanel containerPane = new JPanel();
		containerPane.setLayout(new BorderLayout());
		containerPane.setOpaque(false);
		{

			JToolBar toolbar = new JToolBar();
			toolbar.setLayout(new BoxLayout(toolbar, BoxLayout.X_AXIS));
			toolbar.setBorder(new EmptyBorder(5,5,5,5));
			toolbar.setFloatable(false);
			{
				quantityField.setText((MINE_QUANTITY<100 ? (MINE_QUANTITY<10 ? "00" : "0") : "") + MINE_QUANTITY);
				quantityField.setFont(new Font("sansserif", Font.PLAIN, 20));
				quantityField.setHorizontalAlignment(JTextField.CENTER);
				quantityField.setPreferredSize(new Dimension(50,30));
				quantityField.setOpaque(true);
				quantityField.setForeground(Color.RED);
				quantityField.setBackground(Color.BLACK);
				quantityField.setBorder(new BevelBorder(BevelBorder.RAISED, Color.GRAY, Color.WHITE));
				toolbar.add(quantityField);

				toolbar.add(Box.createGlue());

				refreshButton = new JButton(new ImageIcon(cl.getResource("face-smile.png")));
				refreshButton.setToolTipText(labelName.get("reset"));
				refreshButton.setActionCommand("refresh");
				refreshButton.addActionListener(new MineActionListener());
				toolbar.add(refreshButton);

				JButton resizeButton = new JButton(new ImageIcon(cl.getResource("view-fullscreen.png")));
				resizeButton.setToolTipText(labelName.get("resize"));
				resizeButton.setActionCommand("resize");
				resizeButton.addActionListener(new MineActionListener());
				toolbar.add(resizeButton);

				toolbar.add(Box.createGlue());

				timeField.setText("000");
				timeField.setFont(new Font("sansserif", Font.PLAIN, 20));
				timeField.setHorizontalAlignment(JTextField.CENTER);
				timeField.setPreferredSize(new Dimension(50,30));
				timeField.setOpaque(true);
				timeField.setForeground(Color.RED);
				timeField.setBackground(Color.BLACK);
				timeField.setBorder(new BevelBorder(BevelBorder.RAISED, Color.GRAY, Color.WHITE));
				toolbar.add(timeField);

			}
			containerPane.add(toolbar, BorderLayout.NORTH);

			JPanel mainPane = new JPanel();
			mainPane.setLayout(new BoxLayout(mainPane,BoxLayout.Y_AXIS));
			mainPane.setOpaque(false);
			{
				JPanel fieldPane = new JPanel();
				Dimension fieldPaneSize = new Dimension(FIELD_WIDTH*FIELD_BUTTON_SIZE, FIELD_HEIGHT*FIELD_BUTTON_SIZE);

				fieldPane.setSize(fieldPaneSize);
				fieldPane.setPreferredSize(fieldPaneSize);
				fieldPane.setMaximumSize(fieldPaneSize);
				fieldPane.setMinimumSize(fieldPaneSize);
				fieldPane.setLayout(new GridLayout(FIELD_HEIGHT,FIELD_WIDTH));
				{
					for(int i=1;i<field.length-1;i++){
						for(int j=1;j<field[i].length-1;j++){
							fieldButton[i][j] = new JButton();
							fieldButton[i][j].setFont(new Font("MS UI Gothic", Font.BOLD, 12));
							fieldButton[i][j].setHorizontalAlignment(JLabel.CENTER);
							fieldButton[i][j].setVerticalAlignment(JLabel.CENTER);
							fieldButton[i][j].setMargin(new Insets(0, 0, 0, 0));
							fieldButton[i][j].setName("btn"+i+"_"+j);
							fieldButton[i][j].setActionCommand("btn"+i+"_"+j);
							fieldButton[i][j].addMouseListener(new MineMouseListener());
							fieldButton[i][j].addActionListener(new MineActionListener());
							fieldButton[i][j].setFocusPainted(false);

							fieldPane.add(fieldButton[i][j]);
						}
					}
				}
				mainPane.add(fieldPane);
			}
			containerPane.add(mainPane, BorderLayout.CENTER);
		}

		return containerPane;

	}


	static void resetField(){

		setTime(0);
		t = null;
		t = new PlayTime();

		ClassLoader cl = (new Dummy()).getClass().getClassLoader();
		refreshButton.setIcon(new ImageIcon(cl.getResource("face-smile.png")));
		quantityField.setText((MINE_QUANTITY<100 ? (MINE_QUANTITY<10 ? "00" : "0") :"") + MINE_QUANTITY);
		tempMineQuantity = MINE_QUANTITY;

		for(int i=0;i<field.length;i++){
			for(int j=0;j<field[i].length;j++){

				field[i][j] = 0;
				isCheck[i][j] = false;
				isFlag[i][j] = false;

				if(i>=1 && j>=1){
					if(i<=FIELD_HEIGHT&&j<=FIELD_WIDTH){
						fieldButton[i][j].setText("");
						fieldButton[i][j].setForeground(null);
						fieldButton[i][j].setIcon(null);
						fieldButton[i][j].setBorder((Border) UIManager.getDefaults().get("Button.border"));
						fieldButton[i][j].setBackground((Color) UIManager.getDefaults().get("Button.background"));
					}
				}

			}

		}

		isStart = false;
		isOver = false;
		countOpen = 0;

	}


	static void createField(int x, int y){

		int rnd_x;
		int rnd_y;

		for(int i=0;i<MINE_QUANTITY;i++){
			do{
				rnd_x = (int) (Math.floor(Math.random()*FIELD_HEIGHT)+1);
				rnd_y = (int) (Math.floor(Math.random()*FIELD_WIDTH)+1);
			}while(field[rnd_x][rnd_y]>=10||(rnd_x==x&&rnd_y==y));

			field[rnd_x][rnd_y] = 10;

				for(int j=-1;j<=1;j++){
					for(int k=-1;k<=1;k++){
						field[rnd_x+j][rnd_y+k] += 1;
					}
				}

		}

		isStart = true;
		t.start();

	}


	static void checkField(int x, int y){

		if(!isCheck[x][y]){
			openField(x, y);

			int fieldNumber = field[x][y];
			if(fieldNumber == 0){
				fieldButton[x][y].setIcon(null);
				for(int i=-1;i<=1;i++){
					for(int j=-1;j<=1;j++){
						int next_x = x + i;
						int next_y = y + j;

						if(next_x>=1 && next_y>=1){
							if(next_x<=FIELD_HEIGHT && next_y<=FIELD_WIDTH){
								if(!isCheck[next_x][next_y]){
									checkField(next_x, next_y);
								}
							}
						}
					}
				}
			}
		}
	}

	static void openField(int x, int y){

		ClassLoader cl = (new Dummy()).getClass().getClassLoader();
		isCheck[x][y] = true;
		int fieldNumber = field[x][y];

		if(!(fieldNumber > 10) && !isOver){
			if(x>=1 && y>=1){
				if(x<=FIELD_HEIGHT && y<=FIELD_WIDTH){
					fieldButton[x][y].setHorizontalAlignment(JLabel.CENTER);
					fieldButton[x][y].setVerticalAlignment(JLabel.CENTER);
					fieldButton[x][y].setBackground(new Color(221, 221, 221));
					fieldButton[x][y].setBorder(new LineBorder(new Color(204, 204, 204)));
					countOpen++;
				}
			}
		}

		if(fieldNumber > 10){
			if(!isOver){
				isOver = true;
				gameover(x, y);
			}else{
				fieldButton[x][y].setText("");
				if(!isFlag[x][y]){
					fieldButton[x][y].setIcon(new ImageIcon(cl.getResource("bomb.png")));
				}
			}
		}else if(fieldNumber > 0){
			if(!isOver){
				if(fieldButton[x][y].getIcon()==null){}else{
					tempMineQuantity++;
				}
				isFlag[x][y] = false;
				fieldButton[x][y].setIcon(null);
				fieldButton[x][y].setText(Integer.toString(fieldNumber));

				Color c = null;
				switch(fieldNumber){
				case 1:
					c = Color.BLUE;
					break;

				case 2:
					c = new Color(0,102,0);
					break;

				case 3:
					c = Color.RED;
					break;

				case 4:
					c = new Color(0,0,102);
					break;

				case 5:
					c = new Color(102,0,0);
					break;

				case 6:
					c = new Color(0,153,153);
					break;

				case 7:
					c = new Color(0,0,0);
					break;

				case 8:
					c = new Color(102,102,102);
					break;

				default:
					c = Color.BLACK;
					break;
				}

				fieldButton[x][y].setForeground(c);

				if(FIELD_SIZE - MINE_QUANTITY == countOpen){
					gameclear();
				}

			}else{
				if(isFlag[x][y]){
					fieldButton[x][y].setText("");
					fieldButton[x][y].setIcon(new ImageIcon(cl.getResource("nobomb.png")));
				}
			}
		}else{
			if(fieldButton[x][y].getIcon()==null){}else{
				tempMineQuantity++;
			}
		}

	}


	static void gameclear(){

		ClassLoader cl = (new Dummy()).getClass().getClassLoader();
		t.setOver();
		isOver = true;

		for(int i=1;i<fieldButton.length-1;i++){
			for(int j=1;j<fieldButton[i].length-1;j++){

				isCheck[i][j] = true;
				if(field[i][j]>10){
					fieldButton[i][j].setIcon(new ImageIcon(cl.getResource("flag.png")));
				}

			}

		}

		refreshButton.setIcon(new ImageIcon(cl.getResource("face-smile-big.png")));

	}


	static void gameover(int x, int y){

		for(int i=1;i<field.length-1;i++){
			for(int j=1;j<field[i].length-1;j++){

				openField(i, j);

			}
		}

		ClassLoader cl = (new Dummy()).getClass().getClassLoader();
		refreshButton.setIcon(new ImageIcon(cl.getResource("face-crying.png")));
		fieldButton[x][y].setBackground(Color.RED);
		t.setOver();

	}

	static void setFlag(int x, int y){
		if(isFlag[x][y]){
			isFlag[x][y] = false;
		}else{
			isFlag[x][y] = true;
		}
	}

	static void setTime(){
		int nowTime = t.getTime();

		timeField.setText((nowTime<100 ? (nowTime<10 ? "00" : "0") : "") + nowTime);
	}

	static void setTime(int c){
		int nowTime = c;

		timeField.setText((nowTime<100 ? (nowTime<10 ? "00" : "0") : "") + nowTime);
	}

	void startTime(){
		t.setPause(false);
	}

	void stopTime(){
		t.setPause(true);
	}

	void setVisual(){

		try{
			UIManager.setLookAndFeel(new MyLookAndFeel());
			SwingUtilities.updateComponentTreeUI(this);
		}catch(Exception e){

		}

		/*
		 Font font = new Font("MS UI Gothic",Font.PLAIN,12);
		 LookAndFeel laf = UIManager.getLookAndFeel();
		 Set<Object> keys = laf.getDefaults().keySet();
		 for (Iterator<Object> it = keys.iterator(); it.hasNext();) {
			String key=it.next().toString();
			if (key.indexOf("font") != -1) {
				UIManager.put(key, font);
			}
		 }
		 */

	}

}
