package net.tekiton.minesweeper;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

@SuppressWarnings("serial")
public class Resize extends JFrame {

	static Resize frame = null;
	static Minesweeper ms = null;
	JTextField heightField = null;
	JTextField widthField = null;
	JTextField quantityField = null;
	ClassLoader cl = (new Dummy()).getClass().getClassLoader();

	final static int WIDTH = 20;
	final static int HEIGHT = 15;
	final static int QUANTITY = 10;

	static Locale systemLocale = Locale.getDefault();
	static HashMap<String,String> labelName = new HashMap<String,String>();

	public static void main(String[] args) {
		frame = new Resize();

		ms = new Minesweeper(WIDTH,HEIGHT,QUANTITY);
		ms.setVisible(true);

	}

	Resize(){

		if(systemLocale.getLanguage().equals(new Locale("ja", "", "").getLanguage())){
			labelName.put("title", "盤面の変更");
			labelName.put("easy", "初級");
			labelName.put("normal", "中級");
			labelName.put("hard", "上級");
			labelName.put("height", "高さ（2～52）");
			labelName.put("width", "幅（2～52）");
			labelName.put("quantity", "地雷の数（1～）");
			labelName.put("ok", "　　OK　　");
			labelName.put("cancel", "キャンセル");
		}else{
			labelName.put("title", "Resize");
			labelName.put("easy", "Easy");
			labelName.put("normal", "Normal");
			labelName.put("hard", "Hard");
			labelName.put("height", "Height(2-52)");
			labelName.put("width", "Width(2-52)");
			labelName.put("quantity", "Quantity(1-)");
			labelName.put("ok", "    OK    ");
			labelName.put("cancel", "  Cancel  ");
		}

		setTitle(labelName.get("title"));
		setIconImage(new ImageIcon(cl.getResource("bomb.png")).getImage());
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
		setVisual();

		getContentPane().add(mainPane(), BorderLayout.CENTER);
		pack();

		setLocationRelativeTo(null);

	}

	JPanel mainPane(){

		JPanel mainPane = new JPanel();
		mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.Y_AXIS));
		mainPane.setBorder(new EmptyBorder(new Insets(15, 15, 5, 15)));
		{
			JPanel difficultyPane = new JPanel();
			difficultyPane.setLayout(new GridLayout());
			difficultyPane.setBorder(new EmptyBorder(0,0,5,0));
			{
				JButton easyButton = new JButton(labelName.get("easy"));
				easyButton.setActionCommand("diff_easy");
				easyButton.addActionListener(new ResizeActionListener());
				difficultyPane.add(easyButton);

				JButton normalButton = new JButton(labelName.get("normal"));
				normalButton.setActionCommand("diff_normal");
				normalButton.addActionListener(new ResizeActionListener());
				difficultyPane.add(normalButton);

				JButton hardButton = new JButton(labelName.get("hard"));
				hardButton.setActionCommand("diff_hard");
				hardButton.addActionListener(new ResizeActionListener());
				difficultyPane.add(hardButton);
			}
			mainPane.add(difficultyPane);

			JPanel settingPane = new JPanel();
			settingPane.setLayout(new GridLayout(0,2,0,5));
			{

				JLabel heightLabel = new JLabel(labelName.get("height"));
				settingPane.add(heightLabel);

				heightField = new JTextField(8);
				heightField.setText(Integer.toString(HEIGHT, 10));
				heightField.setHorizontalAlignment(JTextField.RIGHT);
				heightField.setActionCommand("ok");
				heightField.addActionListener(new ResizeActionListener());
				settingPane.add(heightField);

				JLabel widthLabel = new JLabel(labelName.get("width"));
				settingPane.add(widthLabel);

				widthField = new JTextField(8);
				widthField.setText(Integer.toString(WIDTH, 10));
				widthField.setHorizontalAlignment(JTextField.RIGHT);
				widthField.setActionCommand("ok");
				widthField.addActionListener(new ResizeActionListener());
				settingPane.add(widthField);

				JLabel quantityLabel = new JLabel(labelName.get("quantity"));
				settingPane.add(quantityLabel);

				quantityField = new JTextField(8);
				quantityField.setText(Integer.toString(QUANTITY, 10));
				quantityField.setHorizontalAlignment(JTextField.RIGHT);
				quantityField.setActionCommand("ok");
				quantityField.addActionListener(new ResizeActionListener());
				settingPane.add(quantityField);

			}
			mainPane.add(settingPane);

			JPanel buttonPane = new JPanel();
			buttonPane.setBorder(new EmptyBorder(10,0,0,0));
			{
				JButton okButton = new JButton(labelName.get("ok"));
				okButton.setActionCommand("ok");
				okButton.addActionListener(new ResizeActionListener());
				buttonPane.add(okButton);

				JButton cancelButton = new JButton(labelName.get("cancel"));
				cancelButton.setActionCommand("cancel");
				cancelButton.addActionListener(new ResizeActionListener());
				buttonPane.add(cancelButton);
			}
			mainPane.add(buttonPane);

		}

		return mainPane;
	}

	static void closeWindow(){
		System.exit(0);
	}

	void setVisual(){

		try{
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			SwingUtilities.updateComponentTreeUI(this);
		}catch(Exception e){

		}

	}
}
